(*
Andrew OReilly, Stephanie Huang
*)

open Ast
open Sast
open Compile
open Semantics
open Printf

let execute_prog (vdecls, fdecls) = 
  let print_env key (c, s) = 
    print_endline (key ^ " = " ^ (Sast.c_type_str c)) in
  let var_map = List.fold_left 
    (
      fun vmap (t, id) -> 
        if NameMap.mem id vmap
        then raise (Duplicate_name(id ^ " already defined"))
        else NameMap.add id (None, (type_from_str t)) vmap
    )
    NameMap.empty vdecls
  in
  let fun_map = List.fold_left
    (
      fun map fdec -> NameMap.add fdec.fname fdec map
    ) 
    NameMap.empty fdecls
  in
  let globals = {
    variables = var_map;
    functions = fun_map;
  } in
  let scp = {
    parent = None;
    variables = NameMap.empty
  } in
  let env = {
    glob_env = globals;
    scope = scp;
    return_type = SInt;
    return_val = None
  } in
  let _, postexec_env = 
    if NameMap.mem "compose" env.glob_env.functions
    then (), (exec_fun 
      env 
      (NameMap.find "compose" env.glob_env.functions)
      []
    )
    else (print_endline "No compose function"), env
  in let ofile = open_out "play.out" in
  NameMap.iter print_env postexec_env.scope.variables;
  fprintf ofile "#!/bin/bash\njava CSVPlayer out.csv";
  close_out ofile